#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = "Ryan McGuire (ryan@enigmacurry.com)"
__date__   = "Mon Aug 18 14:58:27 2008"


"""A non-feature-complete ical writer"""

from datetime import datetime
import md5
import logging

class Calendar:
    def __init__(self):
        self.name    = "Generic Ical"
        self.domain  = "enigmacurry.com"
        self.version = 1.0
        self.events  = set()

    def __str__(self):
        parts = []
        parts.append("BEGIN:VCALENDAR")
        parts.append("PODID:-//%s//%s//" % (self.name, self.domain))
        parts.append("VERSION:%s" % self.version)
        parts.append("X-WR-CALNAME:%s" % self.name)
        for event in self.events:
            event_str = unicode(event)
            event_str = event_str.encode("utf-8")
            event_parts = event_str.split("\n")
            parts.extend(event_parts)
        parts.append("END:VCALENDAR")
        return "\n".join(parts)

class Event:
    def __init__(self, summary=None):
        self.__dict__['start'] = datetime(1900,1,1)
        self.end               = datetime(1900,1,1)
        self.all_day           = False

        if summary:
            self.summary = summary
        else:
            self.summary = ""
        self.description = ""
        self.location    = ""
        self.information_url = None
        
    def __set_start_date(self, value):
        self.__dict__['start'] = value
        if self.end < self.start:
            #If the event ends before starts,
            #set the end date to be the same
            self.end = value

    def __setattr__(self, name, value):
        if name == "start":
            self.__set_start_date(value)
        else:
            self.__dict__[name] = value

    def generate_uid(self):
        """Generate a UID for this event

        A UID should be a string to uniquely identify this event and should not change if
        event details are changed.

        This seems hard to do, so we'll just base it off the summary and the start day, or
        the information_url and the start day."""

        if self.information_url:
            key = self.information_url + self.start.strftime("%Y%m%d")
        else:
            key = self.summary + self.start.strftime("%Y%m%d")
        logging.debug("key is: '%s'" % key)
        return md5.md5(key.encode("ascii","ignore")).hexdigest()
            
            
    def __str__(self):
        parts = []
        parts.append("BEGIN:VEVENT")
        parts.append("SUMMARY:"+self.summary)
        parts.append("DESCRIPTION:"+self.description)
        parts.append("LOCATION:"+self.location)
        parts.append("UID:"+self.generate_uid())
        if self.all_day:
            parts.append("DTSTART;VALUE=DATE:%s" % (self.start.strftime("%Y%m%d")))
        else:
            parts.append("DTSTART:%s" % (self.start.strftime("%Y%m%dT%H%M%S")))
        if self.all_day:
            parts.append("DTEND;VALUE=DATE:%s" % (self.end.strftime("%Y%m%d")))
        else:
            parts.append("DTEND:%s" % (self.end.strftime("%Y%m%dT%H%M%S")))
        parts.append("END:VEVENT")
        return "\n".join(parts)
